#Requires -RunAsAdministrator

function SafeSaveShortcut($FullName, $Shortcut) {
    $WasReadonly = (Get-ChildItem -Path $FullName).IsReadOnly

    try {
        if ($WasReadOnly) {
            Set-ItemProperty -Path $FullName -Name IsReadOnly -Value $false
        }

        $Shortcut.Save()

        Write-Host "Updated the shortcut $FullName"
    }
    catch {
        Write-Host "Failed to rewrite shortcut: $_"
    }
    finally {
        if ($WasReadOnly -And (-Not (Get-ChildItem $FullName).IsReadOnly)) {
            Set-ItemProperty -Path $FullName -Name IsReadOnly -Value $true
        }
    }
}

function UpdateShortcuts($TaskBarPath, $WorkspaceExePath) {
    $WScriptShell = New-Object -ComObject WScript.Shell
    $UpdatedShortcuts = $false

    Write-Host "Checking for taskbar pins in $TaskBarPath..."

    Get-ChildItem $TaskBarPath -Filter '*.lnk' | ForEach-Object {
        try
        {
            $Shortcut = $WScriptShell.CreateShortcut($_.FullName)

            if ($Shortcut.TargetPath -Like "*workspace*.exe") {
                $Shortcut.TargetPath = $WorkspaceExePath.FullName
                $Shortcut.IconLocation = "$($WorkspaceExePath.FullName),0"
                SafeSaveShortcut $_.FullName $Shortcut
                $UpdatedShortcuts = $true
            }
        }
        catch
        {
            Write-Host "Failed to rewrite shortcut: $_"
        }
    }

    return $UpdatedShortcuts
}

function RestoreTombstonedShortcuts($TombstonePath) {
    $WScriptShell = New-Object -ComObject WScript.Shell
    $RestoredShortcuts = $false

    Write-Host "Restoring tombstoned taskbar pins..."

    Get-ChildItem $TaskBarTombstonePath -Filter '*.lnk' | ForEach-Object {
        try
        {
            $Shortcut = $WScriptShell.CreateShortcut($_.FullName)
            $RestoreShortcut = ($Shortcut.TargetPath -Like "*workspace*.exe")

            if ($RestoreShortcut) {
                Move-Item -Path $_.FullName -Destination $_.Directory.Parent.FullName
                $RestoredShortcuts = $true
            }
        }
        catch
        {
            Write-Host "Failed to restore tombstoned shortcut: $_"
        }
    }

    return $RestoredShortcuts
}

$WorkspaceBasePath = if ($PSScriptRoot) { $PSScriptRoot } else { Get-Location }
Write-Output "Workapce Base Path: '$WorkspaceBasePath'"

$WorkspaceExePath = Get-ChildItem -Path "$WorkspaceBasePath\workspace.exe" | Select-Object -First 1

if (-Not (Test-Path $WorkspaceExePath)) {
    throw "Could not find Workspace executable in '$WorkspaceBasePath'"
}

Write-Output "Workspace Executable: '$WorkspaceExePath'"

Write-Host "Updating taskbar pinned shortcuts"

Get-ChildItem -Path 'HKLM:\SOFTWARE\Microsoft\Windows NT\CurrentVersion\ProfileList' |
        Where-Object { $_.PSChildName.StartsWith('S-1-5-21-') } | ForEach-Object {
    try
    {
        $ProfileImagePath = $_ | Get-ItemPropertyValue -Name 'ProfileImagePath'
        $TaskBarPath = Join-Path $ProfileImagePath "AppData\Roaming\Microsoft\Internet Explorer\Quick Launch\User Pinned\TaskBar"
        $TaskBarTombstonePath = Join-Path $TaskBarPath "Tombstones"

        $UpdatedShortcuts = UpdateShortcuts $TaskBarPath $WorkspaceExePath

        if (-Not $UpdatedShortcuts -And (Test-Path -Path $TaskBarTombstonePath)) {
            $UpdatedShortcuts = RestoreTombstonedShortcuts $TombstonePath

            if ($UpdatedShortcuts) {
                UpdateShortcuts $TaskBarPath $WorkspaceExePath
            }
        }
    }
    catch
    {
        Write-Host "Failed to get ProfileImagePath: $_"
    }
}

$SHChangeNotify = @'
  [System.Runtime.InteropServices.DllImport("Shell32.dll")]
  private static extern int SHChangeNotify(int eventId, int flags, IntPtr item1, IntPtr item2);

  public static void Refresh()  {
      SHChangeNotify(0x8000000, 0x1000, IntPtr.Zero, IntPtr.Zero);
  }
'@

try
{
    Add-Type -MemberDefinition $SHChangeNotify -Namespace WinAPI -Name Explorer
    [WinAPI.Explorer]::Refresh()
}
catch
{
    Write-Host "Failed to refresh icon cache: $_"
}
